#include "atol.h"

static const struct { atom_s ** pAtom; signed char n0; signed char n1; } ATOM_ARG_RULE [] =
{	// ARGS RULES:

	// 2
	&std_atom_set,			-2,	2,
	&std_atom_new,			-2,	2,
	&std_atom_g_new,		-2,	2,
	&std_atom_settag,		-2,	2,

	// 1+
	&std_atom_write,		-1,	1,
	&std_atom_eval,			-1, 1,
	&std_atom_include,		-1,	1,
	&std_atom_var,			-1, 1,
	&std_atom_shell,		-1, 1,
	&std_atom_call,			-1,	1,

	// 0 or 1
	&std_atom_read,			0,	1,
	&std_atom_version,		0,	1,
	&std_atom_atom_num,		0,	1,
	&std_atom_return,		0,	1,
	&std_atom_readchar,		0,	1,
	&std_atom_eol,			0,	1,

	// 0
	&std_atom_nl,			-2,	0,
	&std_atom_about,		-2,	0,
	&std_atom_initialize,	-2,	0,
	&std_atom_trace,		-2,	0,
	&std_atom_notrace,		-2,	0,

	// 1
	&std_atom_sleep,		-2,	1,
	&std_atom_gettag,		-2,	1,
	&std_atom_spy,			-2,	1,

	// 2+
	&std_atom_eq,			-1,	2,
	&std_atom_eqi,			-1,	2,
	&std_atom_eqf,			-1,	2,
	&std_atom_and,			-1,	2,
	&std_atom_or,			-1,	2,
	&std_atom_concat,		-1,	2,
	&std_atom_assert,		-1,	2,

	// 2 or 3
	&std_atom_if,			2,	3,
	&std_atom_right,		2,	3,
	&std_atom_like,			2,	3,
	&std_atom_left,			2,	3,
	&std_atom_at,			2,	3,

	// 3 or 4
	&std_atom_middle,		3,	4,

	// 1 or 2
	&std_atom_not,			1,	2,
	&std_atom_length,		1,	2,
	&std_atom_head,			1,	2,
	&std_atom_tail,			1,	2,
	&std_atom_last,			1,	2,
	&std_atom_reverse,		1,	2,
	&std_atom_rnd,			1,	2,

	// 0 or 7
	&std_atom_gettime,		0,	7,

	// 3+
	&std_atom_append,		-1,	3

};

void after_compile_check_body(list_s *body)
{
	register int n;
	register atom_s *at;
	char *at_name;

	for (register list_s *p = body; p; p = p->next) // CHECK BODY.
	{
		n = list_length(p->sublist);

		at = p->item;
		at_name = at->name.str;

		p->item->set_flag(atom_info_used);

		if (*at_name == '.')
		{
			at = atoms_find(++at_name); // support prefix ignorer.
			if (at == NULL)
			{
				Error(p->sublist != NULL, IDS_CALL_NOT_FUNC, at_name);
				at = p->item;
				at_name = at->name.str;
			}
		}

		// Check argument count errors:

		for (register int j = 0; j < sizeof(ATOM_ARG_RULE)/sizeof(ATOM_ARG_RULE[0]); ++j)
			if (at == *(ATOM_ARG_RULE [j].pAtom))
			{
				char cond [8] = { 0 };

				if (ATOM_ARG_RULE [j].n0 == -2)
				{
					if (n != ATOM_ARG_RULE [j].n1)
						wsprintf(cond, "%i", ATOM_ARG_RULE [j].n1);
				}
				else if (ATOM_ARG_RULE [j].n0 == -1)
				{
					if (n < ATOM_ARG_RULE [j].n1)
						wsprintf(cond, "%i+", ATOM_ARG_RULE [j].n1);
				}
				else 
				{
					if (n != ATOM_ARG_RULE [j].n0 && n != ATOM_ARG_RULE [j].n1)
						wsprintf(cond, "%i or %i", ATOM_ARG_RULE [j].n0, ATOM_ARG_RULE [j].n1);
				}
				Error(*cond, IDS_WRONG_ARGN, at_name, cond);
				break;
			}


		// Check Argument type errors:

		if (at == std_atom_set || at == std_atom_append ||
			(n == 1 && (at == std_atom_read || at == std_atom_readchar) ) ||
			(n == 3 && (at == std_atom_left || at == std_atom_right || at == std_atom_like) ) ||
			(n == 2 && (at == std_atom_not || at == std_atom_length ||
						at == std_atom_head || at == std_atom_tail || at == std_atom_last) ) ||
			(n == 4 && at == std_atom_middle)
		   )
			Error(NULL != p->sublist->sublist, IDS_FIRST_ARG_VAR, at_name);

		else if (at == std_atom_var || at == std_atom_null ||
				 at == std_atom_g_var || at == std_atom_gettime)
		{
			for (register list_s * sb = p->sublist; sb; sb = sb->next)
				Error(NULL != sb->sublist, IDS_ARG_VARS, at_name);
		}

		// Check all parameters:
		after_compile_check_body(p->sublist);
	}
}

void after_compile_work()
{
	extern atom_s * atom_table;
	register atom_s *atom;
	register list_s *p;

#ifdef APP_OPT_DEBUG
	int n = 0;

	console_log(">CHECK..");
#endif

	for (atom = atom_table; atom; atom = atom->next)
	{
		if (atom->notflag(atom_info_external))
		{
			for (p = atom->args; p; p = p->next) // CHECK ARGS.
			{
				Error(p->item->flag(atom_info_function), IDS_NAME_RESERVED_FOR_FUNC);
				p->item->set_flag(atom_info_used);
			}

			if (atom->flag(atom_info_function)) atom->set_flag(atom_info_used);

			after_compile_check_body(atom->body);

		}
#ifdef APP_OPT_DEBUG
		++n;
#endif
	}

#ifdef APP_OPT_DEBUG
	console_log(">ATOM NUM %i", n);
#endif
}
