#include "atol.h"

path_t script_file;
HINSTANCE hInstance;
int iscript;

#if defined(APP_OPT_SAVE_COMPILE) || defined(APP_OPT_LOAD_COMPILE)
path_t compiled_file;			// Filename of file with compiled code.
int icompiled;
#endif

#ifdef APP_OPT_LOG
int ilog;
#endif

int rnd_seed; // for rnd function.

void main()
{
	check_for_help();
	hInstance = GetModuleHandle(NULL);

	rnd_seed = (int) (GetTickCount() ^ GetCurrentThreadId());

#ifdef APP_OPT_DEBUG
	console_log("atol (C) 2005, Alexander G. Karpov [mrhx]\n(!) DEBUG BUILD.");
#endif

#ifdef APP_OPT_LOG
	ilog = file_io_open("atol.log", 'w');
#endif

	setup_interface();
	load_libs();
	locate_targets();
	open_targets();
	compile_targets();
	close_targets();
	run_targets();

#ifdef APP_OPT_DEBUG
	MessageBox(NULL, "Done", "[ atol ]", MB_OK | MB_TOPMOST | MB_ICONINFORMATION);
#endif

#ifdef APP_OPT_LOG
	file_io_close(ilog);
#endif

	ExitProcess(0);
}

void locate_targets() // Locate target script file name and dll's
{
	GetModuleFileName(NULL, script_file, sizeof script_file);
	*get_file_ext_dot(script_file) = 0;
	lstrcat(script_file, ".atol");
#ifdef APP_OPT_DEBUG
	console_log("SCRIPT %s", script_file);
#endif

#ifdef APP_OPT_SAVE_COMPILE
	GetModuleFileName(NULL, compiled_file, sizeof compiled_file);
	*get_file_ext_dot(compiled_file) = 0;
	lstrcat(compiled_file, ".atoc");
#endif
}

char * get_file_ext_dot(char * str) // Returns pointer to the file extension dot or, if no dot, to end of sting.
{
	int n = lstrlen(str);
	for (register int i = n; str [i] != '\\' && i > 0; --i) if (str [i] == '.') return str + i;
	return str + n;
}

void open_targets() // Open program targets.
{
#ifdef APP_OPT_LOAD_COMPILE
	
	if (GetFileAttributes(script_file) != 0xFFFFFFFF)
	{ // script file exists:

#endif

		iscript = file_io_open(script_file, 'r');

	#ifdef APP_OPT_SAVE_COMPILE
		icompiled = file_io_open(compiled_file, 'w');
	#endif

#ifdef APP_OPT_LOAD_COMPILE

	}
	else
	{ // script file does not exist - there must be the compiled file! ::

		iscript = 0;
		icompiled = file_io_open(compiled_file, 'r');
	}

#endif
}

void close_targets() // Close program targets.
{
	if (iscript) file_io_close(iscript);

#if defined(APP_OPT_SAVE_COMPILE) || defined(APP_OPT_LOAD_COMPILE)
	file_io_close(icompiled);
#endif
}

void Error(int IsError, char * Msg)
{
	if (IsError)
	{
		MessageBox(NULL, Msg, "[ atol ] error", MB_OK | MB_TOPMOST | MB_ICONERROR);
		ExitProcess(1);
	}
//#ifdef APP_OPT_DEBUG
//	else console_log("NO_ERR %s", Msg);
//#endif
}

void Error(int IsError, int MsgId, ...)
{
	if (IsError)
	{
		char buf [128], msg [256];

		LoadString(hInstance, MsgId, buf, sizeof(buf));
		wvsprintf(msg, buf, (va_list)(&MsgId+1));

		Error(IsError, msg);
	}
}

void compile_targets() // Compile targets.
{
#ifdef APP_OPT_LOG
	DWORD time_before = GetTickCount();
#endif

	void load_std_atoms();
	load_std_atoms();

#ifdef APP_OPT_LOAD_COMPILE
	if (iscript) engine_compile(iscript, script_file);
	else load_compiled(icompiled);
#else
	engine_compile(iscript, script_file);
#endif

#ifdef APP_OPT_SAVE_COMPILE
	if (iscript) save_compiled(icompiled);
#endif

#ifdef APP_OPT_LOG
	int dtime = GetTickCount() - time_before;
	Log("Compilation time %i.%04i sec", dtime / 1000, (dtime % 1000) * 10);
#endif
}

void run_targets() // Run targets.
{
	engine_run();
}

void * operator new(unsigned int n)
{
	register void * p = (void *) LocalAlloc(LMEM_FIXED, n);
	Error(!p, IDS_NO_MEM, n);
	return p;
}

void operator delete(void * p)
{
	if (p) LocalFree((HLOCAL) p);
}

int s2i(char *s)
{
	register int r = 0;
	register char *p = s;
	int neg = 0;
	// skip possible space:
	while (*p > 0 && *p <= 32) ++p;

	if (*p == '-') { ++neg; ++p; }
	else if (*p == '+') ++p;

	// check number format:
	if (*p == '0')
	{
		++p;
		if (*p == 'x' || *p == 'X')
		{ // hex:
			for (++p; ; ++p)
				if (*p >= '0' && *p <= '9') r = (r << 4) + (*p - '0');
				else if (*p >= 'a' && *p <= 'f') r = (r << 4) + (*p - ('a' - 10));
				else if (*p >= 'A' && *p <= 'F') r = (r << 4) + (*p - ('A' - 10));
				else break;
		}
		else
		{ // oct:
			for (; *p >= '0' && *p <= '7'; ++p) r = (r << 3) + (*p - '0');
		}
	}
	else
	{ // dec:
		for (; *p >= '0' && *p <= '9'; ++p) r = r*10 + (*p - '0');
	}

	return neg?(-r):r;
}

char * i2s(int x, char * s)
{
	wsprintf(s, "%i", x);
	return s;
}

////// RANDOM: >>>>>>>>>>>>>..

unsigned int rnd()
{
	rnd_seed *= 0x4E35;
	return (unsigned int) ++rnd_seed;
}

//////// LOG ////////////////////

#ifdef APP_OPT_LOG

void Log(char *msg, ...)
{
	char str [MAX_PATH];
	file_io_write(ilog, str, wvsprintf(str, msg, (va_list)(&msg + 1)) );
	file_io_write(ilog, "\r\n", 2);
}

#endif

////////// HELP SUPPORT /////////

void check_for_help()
{
	char f [MAX_PATH];
	GetModuleFileName(NULL, f, MAX_PATH);
	if (!lstrcmpi("\\atol.exe", f+lstrlen(f)-9))
	{
		MessageBox(NULL, 
			"Hello.\n\n"
			"You probably have run ATOL at first time.\n"
			"You need done several steps to create your own ATOL application.\n"
			"See atol_manual.htm for help.\n"
			"If you want just to see something in ATOL run several examples.\n\n"
			"Thanks for your interest.\n"
			"Have any questions? Write to mrhx@narod.ru",
			"ATOL: First time run", MB_OK | MB_ICONINFORMATION | MB_TOPMOST);
		ExitProcess(0);
	}
}

