#include "atol.h"

ATOL_INTERFACE_S  IAtol; // ATOL Interface.

ATOL_STRING WINAPI IAtol_Version()                { return ATOL_VERSION; }
ATOL_STRING WINAPI IAtol_Copyright() { return "atol (C) 2005, Alexander G. Karpov [mrhx]"; }

ATOL_ATOM   WINAPI IAtol_Atom(ATOL_STRING name)   { return atoms_add((char *) name); }
ATOL_STRING WINAPI IAtol_AtomName(ATOL_ATOM atom) { return ((atom_s *)atom)->name.str; }

ATOL_STRING WINAPI IAtol_ListItem(ATOL_LIST list) { return ((list_s *)list)->item->name.str; }
ATOL_ATOM   WINAPI IAtol_ListItemAtom(ATOL_LIST list) { return ((list_s *)list)->item; }
ATOL_LIST   WINAPI IAtol_ListNext(ATOL_LIST list) { return ((list_s *)list)->next; }

void	__cdecl IAtol_Printf(ATOL_STRING msg, ...) { console_write((char **)&msg); }
void	WINAPI  IAtol_Print(ATOL_STRING msg) { console_write("%s", msg); }
void	WINAPI  IAtol_Puts(ATOL_STRING msg) { console_write("%s\n", msg); }

int		WINAPI	IAtol_GetC() { return console_read_char(); }
int		WINAPI	IAtol_GetS(ATOL_STRING buf, int maxnum) { return console_read(buf, maxnum); }

int		WINAPI	IAtol_Atoi(ATOL_STRING str) { return s2i((char*) str); }

int		WINAPI	IAtol_ListLen(ATOL_LIST list) { return list_length((list_s *) list); }

ATOL_VAR	WINAPI IAtol_Var(ATOL_ATOM name)	  { return current_scope->find_var((atom_s *) name); }
ATOL_STRING	WINAPI IAtol_VarValue(ATOL_VAR var)	  { return ((atom_s *)(((list_s *)var)->sublist))->name.str; }
ATOL_ATOM	WINAPI IAtol_VarValueAtom(ATOL_VAR var) { return (atom_s*)((list_s *)var)->sublist; }
ATOL_ATOM	WINAPI IAtol_VarSetAtom(ATOL_VAR var, ATOL_ATOM val) { return ((list_s*)var)->sublist = (list_s*)val; }
ATOL_ATOM	WINAPI IAtol_VarSet(ATOL_VAR var, ATOL_STRING val) { return ((list_s*)var)->sublist = (list_s*)atoms_add(val); }

void	WINAPI	IAtol_Error(int isErr, ATOL_STRING msg) { Error(isErr, msg); }

extern atom_s * std_atom_null;

ATOL_ATOM	WINAPI IAtol_Null() { return std_atom_null; }

ATOL_ATOM	WINAPI IAtol_AtomI(int i) { char s[16]; wsprintf(s, "%i", i); return (ATOL_ATOM) atoms_add(s); }
ATOL_ATOM	WINAPI IAtol_ExecAtom(ATOL_ATOM func, ATOL_ATOM *args)
{
	list_s GOAL, *ARGS = NULL;
	atom_s *res;

	GOAL.item = (atom_s *) func;

	if (GOAL.item->notflag(atom_info_function)) // FUNCTION NOT DEFINED!
		return NULL;

	if (args) for (register atom_s **p = (atom_s **) args; *p; ++p) list_add(&ARGS, *p);

	GOAL.next = NULL;
	GOAL.sublist = ARGS;

	res = engine_run_func(&GOAL, 1, 0); // 0 - dont run args.

	if (ARGS) list_free(ARGS);

	return res;
}
ATOL_STRING	WINAPI IAtol_Exec(ATOL_STRING func, ATOL_STRING *args)
{
	list_s GOAL, *ARGS = NULL;
	ATOL_STRING res;

	GOAL.item = atoms_add(func);

	if (GOAL.item->notflag(atom_info_function)) // FUNCTION NOT DEFINED!
		return NULL;

	if (args) for (register ATOL_STRING *p = args; *p; ++p) list_add(&ARGS, *p);

	GOAL.next = NULL;
	GOAL.sublist = ARGS;

	res = engine_run_func(&GOAL, 1, 0)->name.str;

	if (ARGS) list_free(ARGS);

	return res;
}

static list_s * atolCONTEXT = NULL;

void * WINAPI IAtol_GetContextAtom(ATOL_ATOM param)
{
	register list_s *p = list_find(atolCONTEXT, (atom_s *) param);
	if (p) return (void *) p->sublist;
	return NULL;
}

void * WINAPI IAtol_SetContextAtom(ATOL_ATOM param, void *value)
{
	register list_s *p = list_find(atolCONTEXT, (atom_s *) param);
	register void *old = NULL;
	if (p) old = (void *) p->sublist;
	else p = list_add(&atolCONTEXT, (atom_s *) param);
	p->sublist = (list_s *) value;
	return old;
}

void * WINAPI IAtol_GetContext(ATOL_STRING param)
{
	return IAtol_GetContextAtom((ATOL_ATOM) atoms_add(param));
}

void * WINAPI IAtol_SetContext(ATOL_STRING param, void *value)
{
	return IAtol_SetContextAtom((ATOL_ATOM) atoms_add(param), value);
}

ATOL_STRING WINAPI IAtol_ListGet(ATOL_LIST list, int index)
{
	register list_s *p = list_get((list_s *) list, index);
	if (p) return p->item->name.str;
	return NULL;
}
ATOL_ATOM WINAPI IAtol_ListGetAtom(ATOL_LIST list, int index)
{
	register list_s *p = list_get((list_s *) list, index);
	if (p) return p->item;
	return NULL;
}

//Objects:
int WINAPI IAtol_NewGlobalObjAtom(ATOL_ATOM obj_name, ATOL_ATOM obj_cls)
{
	if (obj_class((atom_s *) obj_name)) return 0;
	obj_new((atom_s *) obj_name, (atom_s *) obj_cls);
	return 1;
}

int WINAPI IAtol_NewGlobalObj(ATOL_STRING obj_name, ATOL_STRING obj_class)
{
	return IAtol_NewGlobalObjAtom(atoms_add(obj_name), atoms_add(obj_class));
}

ATOL_ATOM WINAPI IAtol_ClassOfObjAtom(ATOL_ATOM obj_name)
{
	return obj_class((atom_s *) obj_name);
}

ATOL_STRING WINAPI IAtol_ClassOfObj(ATOL_STRING obj_name)
{
	register atom_s *p = obj_class(atoms_add(obj_name));
	if (p) return p->name.str;
	return NULL;
}

ATOL_STRING WINAPI IAtol_ObjProp(ATOL_STRING obj_name, ATOL_STRING prop_name)
{
	return obj_property(atoms_add(obj_name), atoms_add(prop_name))->name.str;
}

ATOL_STRING WINAPI IAtol_ObjSetProp(ATOL_STRING obj_name, ATOL_STRING prop_name, ATOL_STRING val)
{
	return obj_set_property(atoms_add(obj_name), atoms_add(prop_name), atoms_add(val))->name.str;
}

ATOL_ATOM WINAPI IAtol_ObjPropAtom(ATOL_ATOM obj_name, ATOL_ATOM prop_name)
{
	return obj_property((atom_s*) obj_name, (atom_s*) prop_name);
}

ATOL_ATOM WINAPI IAtol_ObjSetPropAtom(ATOL_ATOM obj_name, ATOL_ATOM prop_name, ATOL_ATOM val)
{
	return obj_set_property((atom_s*) obj_name, (atom_s*) prop_name, (atom_s*) val);
}


void setup_interface()
{
#ifdef APP_OPT_DEBUG
	console_log(">SETUP INTERFACE..");
#endif

	IAtol.version		 = IAtol_Version;
	IAtol.copyright		 = IAtol_Copyright;

	IAtol.atom			 = IAtol_Atom;
	IAtol.atom_name		 = IAtol_AtomName;

	IAtol.list_item		 = IAtol_ListItem;
	IAtol.list_item_atom = IAtol_ListItemAtom;
	IAtol.list_next		 = IAtol_ListNext;

	IAtol.printf		 = IAtol_Printf;
	IAtol.print			 = IAtol_Print;
	IAtol.puts			 = IAtol_Puts;

	IAtol.getc			 = IAtol_GetC;
	IAtol.gets			 = IAtol_GetS;

	IAtol.atoi			 = IAtol_Atoi;

	IAtol.list_length	 = IAtol_ListLen;

	IAtol.var			 = IAtol_Var;
	IAtol.var_value		 = IAtol_VarValue;
	IAtol.var_value_atom = IAtol_VarValueAtom;
	IAtol.var_set		 = IAtol_VarSet;
	IAtol.var_set_atom	 = IAtol_VarSetAtom;

	IAtol.error			 = IAtol_Error;
	IAtol.null			 = IAtol_Null;

	IAtol.atomi			 = IAtol_AtomI;
	IAtol.exec			 = IAtol_Exec;
	IAtol.exec_atom		 = IAtol_ExecAtom;

	IAtol.get_context	 = IAtol_GetContext;
	IAtol.set_context	 = IAtol_SetContext;
	IAtol.get_context_atom = IAtol_GetContextAtom;
	IAtol.set_context_atom = IAtol_SetContextAtom;

	IAtol.list_get		 = IAtol_ListGet;
	IAtol.list_get_atom  = IAtol_ListGetAtom;

	// Objects:
	IAtol.new_global_obj		= IAtol_NewGlobalObj;
	IAtol.new_global_obj_atom	= IAtol_NewGlobalObjAtom;
	IAtol.class_of_obj			= IAtol_ClassOfObj;
	IAtol.class_of_obj_atom		= IAtol_ClassOfObjAtom;
	IAtol.obj_prop				= IAtol_ObjProp;
	IAtol.obj_set_prop			= IAtol_ObjSetProp;
	IAtol.obj_prop_atom			= IAtol_ObjPropAtom;
	IAtol.obj_set_prop_atom		= IAtol_ObjSetPropAtom;


#ifdef APP_OPT_DEBUG
	console_log(">END");
#endif

}
