#ifndef atol_interface_h__included
#define atol_interface_h__included

// ATOL (C) 2005, Alexander G. Karpov [mrhx]
// Version 0.9
// http://mrhx.narod.ru/
// (!)    ATOL   .
// (!)        ATOL.

#include <windows.h>

typedef void * ATOL_ATOM;					// ATOM TYPE.
typedef char * ATOL_STRING;					// STRING TYPE.
typedef void * ATOL_LIST;					// LIST TYPE.
typedef void * ATOL_VAR;					// VARIABLE TYPE.

typedef struct ATOL_INTERFACE_S
{
	ATOL_STRING (WINAPI * version)();					 // Get version string.
	ATOL_STRING (WINAPI * copyright)();					 // Get copyright string.

	ATOL_ATOM (WINAPI * atom)(ATOL_STRING name);		 // Add an atom or get existing one.
	ATOL_STRING (WINAPI * atom_name)(ATOL_ATOM atom);	 // Get atom name.

	ATOL_STRING (WINAPI * list_item)(ATOL_LIST list);	 // Get list item.
	ATOL_ATOM (WINAPI * list_item_atom)(ATOL_LIST list); // Get list item as atom.
	ATOL_LIST (WINAPI * list_next)(ATOL_LIST list);		 // Get next list element.

	void (__cdecl * printf)(ATOL_STRING msg, ...);		 // Write a formatted string to console.
	void (WINAPI * print)(ATOL_STRING msg);				 // Write a string to console.
	void (WINAPI * puts)(ATOL_STRING msg);				 // Write a string to console and end line.

	int (WINAPI * getc)();								 // Read a character from console.
	int (WINAPI * gets)(ATOL_STRING buf, int maxnum);	 // Read a string from console.

	int (WINAPI * atoi)(ATOL_STRING str);				 // Convert string to integer.

	int (WINAPI * list_length)(ATOL_LIST list);			 // List length.

	ATOL_VAR (WINAPI * var)(ATOL_ATOM name);			 // Get variable by its name.
	ATOL_STRING (WINAPI * var_value)(ATOL_VAR var);		 // Get value of a variable.
	ATOL_ATOM (WINAPI * var_value_atom)(ATOL_VAR var);	 // Get value of a variable as atom.
	ATOL_ATOM (WINAPI * var_set)(ATOL_VAR var, ATOL_STRING val); // Set value of a variable.
	ATOL_ATOM (WINAPI * var_set_atom)(ATOL_VAR var, ATOL_ATOM val); // Set value of a variable.

	void (WINAPI * error)(int is_err, ATOL_STRING msg);	 // If is_err, show msg and quit program.

	ATOL_ATOM (WINAPI * null)();						 // Returns empty atom ('').

// --- v0.9 ---

	ATOL_ATOM (WINAPI * atomi)(int i);					 // Get atom from integer.

	ATOL_STRING (WINAPI * exec)(ATOL_STRING func, ATOL_STRING *args); // Call a function.
	ATOL_ATOM (WINAPI * exec_atom)(ATOL_ATOM func, ATOL_ATOM *args); // Call a function.

	void * (WINAPI * get_context_atom)(ATOL_ATOM param);			 // Get context parameter value.
	void * (WINAPI * set_context_atom)(ATOL_ATOM param, void * value); // Set context parameter (returns old value of param).
	void * (WINAPI * get_context)(ATOL_STRING param);			 // Get context parameter value.
	void * (WINAPI * set_context)(ATOL_STRING param, void * value); // Set context parameter (returns old value of param).
	// Context is a list of pairs PARAM=VALUE.
	// External library can set param of context or get its value.

	ATOL_STRING (WINAPI * list_get)(ATOL_LIST list, int index);	   // Get list item by index.
	ATOL_ATOM (WINAPI * list_get_atom)(ATOL_LIST list, int index);  // Get list item by index.

	// Functions to define global objects and work with it:
	// ------------
	// Global object is just an item of list of global objects.
	// Object belongs to a class.
	int (WINAPI * new_global_obj)(ATOL_STRING obj_name, ATOL_STRING class_name); // If result != 0 - object is created, otherwise it is already exists.
	int (WINAPI * new_global_obj_atom)(ATOL_ATOM obj_name, ATOL_ATOM class_name);

	ATOL_STRING (WINAPI * class_of_obj)(ATOL_STRING obj_name);  // Get class name of object.
	ATOL_ATOM (WINAPI * class_of_obj_atom)(ATOL_ATOM obj_name);

	// get value of object property:
	ATOL_STRING (WINAPI * obj_prop)(ATOL_STRING obj_name, ATOL_STRING prop_name);
	ATOL_ATOM (WINAPI * obj_prop_atom)(ATOL_ATOM obj_name, ATOL_ATOM prop_name);
	// set value of object property:
	ATOL_STRING (WINAPI * obj_set_prop)(ATOL_STRING obj_name, ATOL_STRING prop_name, ATOL_STRING prop_val); 
	ATOL_ATOM (WINAPI * obj_set_prop_atom)(ATOL_ATOM obj_name, ATOL_ATOM prop_name, ATOL_ATOM prop_val);


} *ATOL_INTERFACE;


#endif // atol_interface_h__included
