/////////////////////////////////////////////////
//  INCLUDED FILE INTO STD_FUNC.CPP  ////////////
/////////////////////////////////////////////////

/*
	EXPRESSION ::= SIMPLE REST
	REST ::= '+' EXPRESSION | '-' EXPRESSION | '*' SIMPLE REST | '/' SIMPLE REST
	SIMPLE = NUMBER | '(' EXPRESSION ')' | ! SIMPLE
*/

typedef int eval_res_t;

eval_res_t mpow(eval_res_t x, int p)
{
	eval_res_t r = 1;
	do
	{
		if (p & 1) r *= x;
		x *= x;
		p = p >> 1;
	}
	while (p);
	return r;
}

void __fastcall eval_skip_space(const char **pp)
{
	for (; **pp > 0 && **pp <= 32; ++ *pp);
}

void __fastcall eval_expr(const char **pp, eval_res_t *res);
void __fastcall eval_rest(const char **pp, eval_res_t *res);

void __fastcall eval_simple(const char **pp, eval_res_t *res)
{
	eval_skip_space(pp);
	if (**pp == '(')
	{
		++ *pp;
		eval_expr(pp, res);
		eval_skip_space(pp);
		Error(**pp != ')', IDS_EXPECTED, ")");
		++ *pp;
	}
	else if (**pp == '!')
	{
		++ *pp;
		eval_simple(pp, res);
		(*res) = ~ (*res);
	}
	else
	{
		*res = 0;

		if (**pp == '0')
		{
			++ *pp;
			if (**pp == 'x' || **pp == 'X')
			{ // hex:
				for (++ *pp; ; ++ *pp)
					if (**pp >= '0' && **pp <= '9') *res = (*res << 4) + (**pp - '0');
					else if (**pp >= 'a' && **pp <= 'f') *res = (*res << 4) + (**pp - ('a' - 10));
					else if (**pp >= 'A' && **pp <= 'F') *res = (*res << 4) + (**pp - ('A' - 10));
					else break;
			}
			else
			{ // oct:
				for (; **pp >= '0' && **pp <= '7'; ++ *pp) *res = (*res << 3) + (**pp - '0');
			}
		}
		else
		{ // dec:
			for (; **pp >= '0' && **pp <= '9'; ++ *pp) *res = *res * 10 + (**pp - '0');
		}
	}
}

// OPERATIONS:

void __fastcall eval_op_func_void(const char **pp, eval_res_t *res) {}

void __fastcall eval_op_func_plus(const char **pp, eval_res_t *res)
{
	eval_res_t r = 0;
	++ *pp;
	eval_expr(pp, &r);
	*res += r;
}

void __fastcall eval_op_func_gr(const char **pp, eval_res_t *res)
{
	eval_res_t r = 0;
	++ *pp;
	eval_expr(pp, &r);
	*res = *res > r;
}

void __fastcall eval_op_func_ls(const char **pp, eval_res_t *res)
{
	eval_res_t r = 0;
	++ *pp;
	eval_expr(pp, &r);
	*res = *res < r;
}

void __fastcall eval_op_func_eq(const char **pp, eval_res_t *res)
{
	eval_res_t r = 0;
	++ *pp;
	eval_expr(pp, &r);
	*res = (*res == r);
}

void __fastcall eval_op_func_or(const char **pp, eval_res_t *res)
{
	eval_res_t r = 0;
	++ *pp;
	eval_expr(pp, &r);
	*res |= r;
}

void __fastcall eval_op_func_xor(const char **pp, eval_res_t *res)
{
	eval_res_t r = 0;
	++ *pp;
	eval_expr(pp, &r);
	*res ^= r;
}

void __fastcall eval_op_func_minus(const char **pp, eval_res_t *res)
{
	eval_res_t r = 0;
	++ *pp;
	eval_expr(pp, &r);
	*res -= r;
}

void __fastcall eval_op_func_and(const char **pp, eval_res_t *res)
{
	eval_res_t r = 0;
	++ *pp;
	eval_simple(pp, &r);
	*res &= r;
	eval_rest(pp, res);
}

void __fastcall eval_op_func_mul(const char **pp, eval_res_t *res)
{
	eval_res_t r = 0;
	++ *pp;
	eval_simple(pp, &r);
	*res *= r;
	eval_rest(pp, res);
}

void __fastcall eval_op_func_div(const char **pp, eval_res_t *res)
{
	eval_res_t r = 0;
	++ *pp;
	eval_simple(pp, &r);
	*res /= r;
	eval_rest(pp, res);
}

void __fastcall eval_op_func_mod(const char **pp, eval_res_t *res)
{
	eval_res_t r = 0;
	++ *pp;
	eval_simple(pp, &r);
	*res = *res % r;
	eval_rest(pp, res);
}

void __fastcall eval_op_func_pow(const char **pp, eval_res_t *res)
{
	eval_res_t r = 0;
	++ *pp;
	eval_simple(pp, &r);
	*res = mpow(*res, r);
	eval_rest(pp, res);
}


typedef void (__fastcall *EVAL_OP_FUNC_T)(const char**, eval_res_t*);

EVAL_OP_FUNC_T eval_op_func [256]; // ALL OPERATIONS.


void eval_init()
{
	for (register int i = 0; i < 256; ++i) eval_op_func [i] = eval_op_func_void;

	eval_op_func ['+']  = eval_op_func_plus;
	eval_op_func ['>']  = eval_op_func_gr;
	eval_op_func ['<']  = eval_op_func_ls;
	eval_op_func ['=']  = eval_op_func_eq;
	eval_op_func ['|']  = eval_op_func_or;
	eval_op_func ['~']  = eval_op_func_xor;
	eval_op_func ['-']  = eval_op_func_minus;
	eval_op_func ['&']  = eval_op_func_and;
	eval_op_func ['*']  = eval_op_func_mul;
	eval_op_func ['/']  = eval_op_func_div;
	eval_op_func ['\\'] = eval_op_func_mod;
	eval_op_func ['^']  = eval_op_func_pow;
}

void __fastcall eval_rest(const char **pp, eval_res_t *res)
{
	eval_res_t r = 0;
	eval_skip_space(pp);

	(*(eval_op_func [**pp]))(pp, res);
}

void __fastcall eval_expr(const char **pp, eval_res_t *res)
{
	eval_simple(pp, res);
	eval_rest(pp, res);
}

eval_res_t evaluate(const char *str)
{
	eval_res_t res = 0;
	eval_expr(&str, &res);
	return res;
}

atol_func(atol_eval)
{
	STR_S(expr);
	for (register list_s *p = Args; p; p = p->next) expr.concat(p->item->name);

	STR_S_(res, evaluate(expr.str));
	expr.free();

	return atoms_add(&res);
}

//---------------------------------
// set(y 5)
// set(x eval(y + 10 * 2))
// writenl('x =' x) % x = 25
//---------------------------------

