#include "atol.h"

//-----------------------------------------------------------------------------------
//  This module contents logic to load external (foreign) libraries (*.dll).
//-----------------------------------------------------------------------------------

// INSTALL FUNCTION :: const char ** WINAPI atol_lib_install(ATOL_INTERFACE IAtol);
//
// EXTERNAL FUNCTION :: ATOL_ATOM WINAPI Function(ATOL_INTERFACE IAtol, ATOL_LIST Args, void * Reserved);
//

#define EXTERN_LIB_INSTALL		"atol_lib_install"

void load_libs()
{
	WIN32_FIND_DATA fd;
	HANDLE hFind = FindFirstFile("*.dll", &fd);

	if (hFind == INVALID_HANDLE_VALUE)
	{
#ifdef APP_OPT_DEBUG
		console_log(">NO LIBS");
#endif
		return;
	}
#ifdef APP_OPT_DEBUG
	console_log(">LIBS");
#endif

	HMODULE hDll;
	void *Install, *Func;
	const char ** Export;

	do
	{
#ifdef APP_OPT_DEBUG
		console_log("? %s", fd.cFileName);
#endif
		if ((hDll = LoadLibrary(fd.cFileName)) &&
			(Install = GetProcAddress(hDll, EXTERN_LIB_INSTALL)) )
		{

#ifdef APP_OPT_DEBUG
			console_log("+");
#endif
			Export = (* (const char ** (WINAPI *)(ATOL_INTERFACE)) Install)(&IAtol);

			if (Export)
			{
				for (; *Export; Export += 2)
				{
#ifdef APP_OPT_DEBUG
					console_log("# %s %s", *Export, Export[1]);
#endif
					Func = GetProcAddress(hDll, Export [1]);
					if (Func)
					{
						register_extern((char *) *Export, Func);
					}
#ifdef APP_OPT_DEBUG
					else console_log(">NO %s", Export[1]);
#endif
				}
			}
			else
			{

#ifdef APP_OPT_DEBUG
				console_log("EXPORT NULL");
#endif
				FreeLibrary(hDll);
			}
		}
		else
		{

#ifdef APP_OPT_DEBUG
			console_log("-");
#endif
			FreeLibrary(hDll);
		}
	}
	while (FindNextFile(hFind, &fd));

	FindClose(hFind);

#ifdef APP_OPT_DEBUG
	console_log(">END");
#endif
}
