#include "atol.h"

#ifdef APP_OPT_SAVE_COMPILE

extern atom_s * atom_table;

void save_compiled_list(int f, list_s *p)
{
	int zero = 0; // 32-bit zero.

	for (; p; p = p->next)
	{
		Error(!(p->item), "internal: NULL list item");
		file_io_write_encrypted(f, &(p->item), 4);

		save_compiled_list(f, p->sublist); // save sublist.
	}
	file_io_write_encrypted(f, &zero, 4);	// write end of list.
}

void save_compiled(int f)
{
#ifdef APP_OPT_DEBUG
	console_log(">SAVE..");
#endif

	int zero = 0; // 32-bit zero.
	int one = 1; // 32-bit one.
	int two = 2;
	int three = 3;
	char *version = ATOL_VERSION;

	crypt_start(); // start encrypted file IO.

	file_io_write_encrypted(f, "ATOL", 4);	// write signature.
	file_io_write_encrypted(f, version, lstrlen(version)+1); // write version.

	for (atom_s *p = atom_table; p; p = p->next) // WRITE ATOM TABLE
		if (p->flag(atom_info_used))
		{
			file_io_write_encrypted(f, &p, 4); // write atom ID.
			file_io_write_encrypted(f, p->name.str, p->name.len+1); // write atom.
		}
	file_io_write_encrypted(f, &zero, 4); // write end of atom table.

	for (p = atom_table; p; p = p->next)
		if (p->flag(atom_info_used))
		{
			if (p->flag(atom_info_external))
			{
				//file_io_write_encrypted(f, &zero, 4); // NO ARGS
				//file_io_write_encrypted(f, &zero, 4); // AND NO BODY
				file_io_write_encrypted(f, &one, 4); // no args and no body (external).
			}
			else
			{
				if (!p->args && !p->body)
				{
					if (p->flag(atom_info_function))
						file_io_write_encrypted(f, &two, 4); // no args and no body (function).
					else
						file_io_write_encrypted(f, &three, 4); // no args and no body (just an atom).
				}
				else
				{
					if (p->args) Error((int) p->args->item >= 1 && (int) p->args->item <= 3, "internal: args in 1..3");
					if (p->body) Error((int) p->body->item >= 1 && (int) p->body->item <= 3, "internal: body in 1..3");

					save_compiled_list(f, p->args); // save args.
					save_compiled_list(f, p->body); // save body.
				}
			}
		}

	crypt_finish();

#ifdef APP_OPT_DEBUG
	console_log(">SAVED");
#endif
}

#endif

 //////// LOAD COMPILED FILE ///////////////////

#ifdef APP_OPT_LOAD_COMPILE

atom_s * find_compiled_atom(list_s *ATOMS, int id)
{
	for (register list_s *p = ATOMS; p; p = p->next)
		if (p->sublist == (list_s *) id ) return p->item;
	Error(1, "internal: atom id not found");
	return NULL;
}

list_s * open_compiled_list(int f, list_s *ATOMS, int tab = 0)
{
	list_s *p = NULL;
	int id = 0;
	register atom_s *at;

	file_io_read_encrypted(f, &id, 4);

	if (id >= 1 && id <= 3) return (list_s *) id;

	while (id)
	{
		at = find_compiled_atom(ATOMS, id);

#ifdef APP_OPT_LOG
		char str [1024];
		memset(str, 32, tab);
		str [tab] = 0;
		lstrcat(str, at->name);
		Log("%s %i", str, id);
#endif
		list_add(&p, at)->sublist = open_compiled_list(f, ATOMS, tab+1);

		file_io_read_encrypted(f, &id, 4);
	}
	return p;
}

void load_compiled(const char * fname)
{
	int file = file_io_open((char *) fname, 'r');
	load_compiled(file);
	file_io_close(file);
}

void load_compiled(int f)
{
#ifdef APP_OPT_DEBUG
	console_log(">LOAD..");
#endif

	char buf [8];
	list_s *ATOMS = NULL; // additional list of atoms.
	int id = 0;
	STR_S(atom);
	
#ifdef APP_OPT_LOG
	Log("Loading compiled..");
#endif

	crypt_start();

	file_io_read_encrypted(f, buf, 4); buf [4] = 0; // read signature.
	Error(lstrcmp(buf, "ATOL"), IDS_WRONG_SIGNATURE);

	file_io_read_encrypted_str(f, buf); // read version.
	if (lstrcmp(buf, ATOL_VERSION))
		MessageBox(NULL, "This version of ATOL is not equal to version of the compiled file.\nIt may be inconsistent!", "[ atol ] WARNING", MB_OK | MB_ICONWARNING | MB_TOPMOST);

#ifdef APP_OPT_LOG
	Log("Version '%s'", buf);
#endif

	// read atoms:
	file_io_read_encrypted(f, &id, 4); // read atom ID.
	while (id) // while there is next atom...
	{
		file_io_read_encrypted_str(f, atom); // read next atom

#ifdef APP_OPT_LOG
		Log("Atom 0x%08X '%s'", id, atom.str);
#endif

		list_add(&ATOMS, atoms_add(atom.str)) ->sublist = (list_s *) id; // store ID in 'sublist' field.

		file_io_read_encrypted(f, &id, 4); // read next atom ID.
	}

	atom.free();

	for (list_s *p = ATOMS; p; p = p->next)
	{
		register atom_s *at = p->item;

#ifdef APP_OPT_LOG
		Log("%s", at->name.str);
#endif
		// read args:
		register list_s *tmp = open_compiled_list(f, ATOMS, 1);

		switch ((int) tmp)
		{
		case 1: // external:
			Error(at->notflag(atom_info_external), IDS_NO_EXTERN, at->name.str);
			break;

		case 2: // function:
			at->set_flag(atom_info_function);
			break;

		case 3: // just an atom:
			break;

		default:
			if (tmp) at->args = tmp;
			// read body:
			if (tmp = open_compiled_list(f, ATOMS, 1)) at->body = tmp;
			if (at->args || at->body) at->set_flag(atom_info_function);
		}

	}

	crypt_finish();

#ifdef APP_OPT_LOG
	Log("Loaded");
#endif

#ifdef APP_OPT_DEBUG
	console_log(">LOADED");
#endif
}

#endif
